package com.example.jpa;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import javax.sql.DataSource;

/**
 * Session Bean testing the performance of several ways to call SELECT *
 */
@Stateless
public class JPAPerfTestBean implements JPAPerfTestBeanLocal {

	private static Logger LOG =  Logger.getLogger(JPAPerfTestBean.class.getName());
	
	EntityManager em = null;

	@PersistenceUnit(unitName="testpu")
	EntityManagerFactory emf;
	
	@Resource(mappedName="jdbc/__default") 
    DataSource dataSource;

	private Connection jdbcConnection;
	
	Query queryJPA = null;
	Query queryEJB = null;
	PreparedStatement stmt1 = null;

	@PostConstruct 
	public void init() 
	{
	  em = emf.createEntityManager();
	  try {
		jdbcConnection = dataSource.getConnection();
	  } catch (SQLException e) {
		e.printStackTrace();
	  }
      prepareQueries();
	}
	
	@PreDestroy public void destroy()
	{
	  em.close();
	}
	
    public void prepareQueries()
    {
    	String statement = "SELECT * from customer";
    	queryJPA = em.createNativeQuery(statement);
    	queryJPA.setHint("eclipselink.jdbc.fetch-size", 100);
    	queryJPA.setHint("eclipselink.jdbc.read-only", true); 
    	try {
			stmt1 = jdbcConnection.prepareStatement(statement, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
		} catch (SQLException e) {
			e.printStackTrace();
		}
        queryEJB = em.createNativeQuery("SELECT C.* FROM CUSTOMER C", Customer.class);
    }
    
    public void doQueryJPA()
    {
    	LOG.info("start JPA");
    	for (int i=0; i<1000; i++)
    	{
    		queryJPA.getResultList();
    	}
    	LOG.info("finish JPA");
    }

    public void doQueryJDBC()
    {
    	LOG.info("start JDBC");
    	for (int i=0; i<1000; i++)
    	{
    		try {
				stmt1.executeQuery();
			} catch (SQLException e) {
				e.printStackTrace();
			}
    	}
    	LOG.info("finish JDBC");
    }
    
    public void doQueryEJB()
    {
    	LOG.info("start EJB");
        for (int i=0; i<1000; i++)
    	{
    		queryEJB.getResultList();
    	}
    	LOG.info("finish EJB");
    }    
}
